@@ -1,27 +1,17 @@
 """All internal ansible-lint rules."""
+from __future__ import annotations
+
 import copy
-import glob
-import importlib.util
 import inspect
 import logging
-import os
 import re
+import sys
 from argparse import Namespace
 from collections import defaultdict
 from functools import lru_cache
-from importlib.abc import Loader
-from typing import (
-    Any,
-    Dict,
-    Iterator,
-    List,
-    MutableMapping,
-    MutableSequence,
-    Optional,
-    Set,
-    Union,
-    cast,
-)
+from importlib import import_module
+from pathlib import Path
+from typing import Any, Iterable, Iterator, MutableMapping, MutableSequence, cast
 
 from ruamel.yaml.comments import CommentedMap, CommentedSeq
 
@@ -33,9 +23,11 @@ from ansiblelint._internal.rules import (
     BaseRule,
     LoadingFailureRule,
     RuntimeErrorRule,
+    WarningRule,
 )
 from ansiblelint.config import PROFILES, get_rule_config
 from ansiblelint.config import options as default_options
+from ansiblelint.constants import LINE_NUMBER_KEY, RULE_DOC_URL, SKIPPED_RULES_KEY
 from ansiblelint.errors import MatchError
 from ansiblelint.file_utils import Lintable, expand_paths_vars
 
@@ -56,7 +48,12 @@ class AnsibleLintRule(BaseRule):
     """AnsibleLintRule should be used as base for writing new rules."""
 
     @property
-    def rule_config(self) -> Dict[str, Any]:
+    def url(self) -> str:
+        """Return rule documentation url."""
+        return RULE_DOC_URL + self.id + "/"
+
+    @property
+    def rule_config(self) -> dict[str, Any]:
         """Retrieve rule specific configuration."""
         return get_rule_config(self.id)
 
@@ -65,10 +62,6 @@ class AnsibleLintRule(BaseRule):
         """Return a configured value for given key string."""
         return self.rule_config.get(key, None)
 
-    def __repr__(self) -> str:
-        """Return a AnsibleLintRule instance representation."""
-        return self.id + ": " + self.shortdesc
-
     @staticmethod
     def unjinja(text: str) -> str:
         """Remove jinja2 bits from a string."""
@@ -80,10 +73,10 @@ class AnsibleLintRule(BaseRule):
     # pylint: disable=too-many-arguments
     def create_matcherror(
         self,
-        message: Optional[str] = None,
+        message: str | None = None,
         linenumber: int = 1,
         details: str = "",
-        filename: Optional[Union[str, Lintable]] = None,
+        filename: Lintable | None = None,
         tag: str = "",
     ) -> MatchError:
         """Instantiate a new MatchError."""
@@ -98,7 +91,7 @@ class AnsibleLintRule(BaseRule):
             match.tag = tag
         # search through callers to find one of the match* methods
         frame = inspect.currentframe()
-        match_type: Optional[str] = None
+        match_type: str | None = None
         while not match_type and frame is not None:
             func_name = frame.f_code.co_name
             match_type = match_types.get(func_name, None)
@@ -109,13 +102,21 @@ class AnsibleLintRule(BaseRule):
             frame = frame.f_back  # get the parent frame for the next iteration
         return match
 
-    def matchlines(self, file: "Lintable") -> List[MatchError]:
-        matches: List[MatchError] = []
-        if not self.match:
-            return matches
+    @staticmethod
+    def _enrich_matcherror_with_task_details(
+        match: MatchError, task: dict[str, Any]
+    ) -> None:
+        match.task = task
+        if not match.details:
+            match.details = "Task/Handler: " + ansiblelint.utils.task_to_str(task)
+        if match.linenumber < task[LINE_NUMBER_KEY]:
+            match.linenumber = task[LINE_NUMBER_KEY]
+
+    def matchlines(self, file: Lintable) -> list[MatchError]:
+        matches: list[MatchError] = []
         # arrays are 0-based, line numbers are 1-based
         # so use prev_line_no as the counter
-        for (prev_line_no, line) in enumerate(file.content.split("\n")):
+        for prev_line_no, line in enumerate(file.content.split("\n")):
             if line.lstrip().startswith("#"):
                 continue
 
@@ -138,74 +139,98 @@ class AnsibleLintRule(BaseRule):
             matches.append(matcherror)
         return matches
 
-    def matchtasks(self, file: Lintable) -> List[MatchError]:
-        matches: List[MatchError] = []
+    # pylint: disable=too-many-branches
+    def matchtasks(self, file: Lintable) -> list[MatchError]:  # noqa: C901
+        """Call matchtask for each task inside file and return aggregate results.
+
+        Most rules will never need to override matchtasks because its main
+        purpose is to call matchtask for each task/handlers in the same file,
+        and to aggregate the results.
+        """
+        matches: list[MatchError] = []
         if (
-            not self.matchtask
-            or file.kind not in ["handlers", "tasks", "playbook"]
+            file.kind not in ["handlers", "tasks", "playbook"]
             or str(file.base_kind) != "text/yaml"
         ):
             return matches
 
-        tasks_iterator = ansiblelint.yaml_utils.iter_tasks_in_file(file, self.id)
-        for raw_task, task, skipped, error in tasks_iterator:
+        tasks_iterator = ansiblelint.yaml_utils.iter_tasks_in_file(file)
+        for raw_task, task, skipped_tags, error in tasks_iterator:
             if error is not None:
                 # normalize_task converts AnsibleParserError to MatchError
                 return [error]
 
-            if skipped or "action" not in task:
+            if (
+                self.id in skipped_tags
+                or ("action" not in task)
+                or "skip_ansible_lint" in task.get("tags", [])
+            ):
                 continue
 
             if self.needs_raw_task:
                 task["__raw_task__"] = raw_task
 
             result = self.matchtask(task, file=file)
-
             if not result:
                 continue
 
-            message = None
-            if isinstance(result, str):
-                message = result
-            task_msg = "Task/Handler: " + ansiblelint.utils.task_to_str(task)
-            match = self.create_matcherror(
-                message=message,
-                linenumber=task[ansiblelint.utils.LINE_NUMBER_KEY],
-                details=task_msg,
-                filename=file,
-            )
-            match.task = task
+            if isinstance(result, Iterable) and not isinstance(
+                result, str
+            ):  # list[MatchError]
+                # https://github.com/PyCQA/pylint/issues/6044
+                # pylint: disable=not-an-iterable
+                for match in result:
+                    if match.tag in skipped_tags:
+                        continue
+                    self._enrich_matcherror_with_task_details(match, task)
+                    matches.append(match)
+                continue
+            if isinstance(result, MatchError):
+                if result.tag in skipped_tags:
+                    continue
+                match = result
+            else:  # bool or string
+                message = None
+                if isinstance(result, str):
+                    message = result
+                match = self.create_matcherror(
+                    message=message,
+                    linenumber=task[LINE_NUMBER_KEY],
+                    filename=file,
+                )
+
+            self._enrich_matcherror_with_task_details(match, task)
             matches.append(match)
         return matches
 
-    def matchyaml(self, file: Lintable) -> List[MatchError]:
-        matches: List[MatchError] = []
-        if not self.matchplay or str(file.base_kind) != "text/yaml":
+    def matchyaml(self, file: Lintable) -> list[MatchError]:
+        matches: list[MatchError] = []
+        if str(file.base_kind) != "text/yaml":
             return matches
 
-        yaml = ansiblelint.utils.parse_yaml_linenumbers(file)
+        yaml = file.data
         # yaml returned can be an AnsibleUnicode (a string) when the yaml
         # file contains a single string. YAML spec allows this but we consider
         # this an fatal error.
         if isinstance(yaml, str):
             if yaml.startswith("$ANSIBLE_VAULT"):
                 return []
-            return [MatchError(filename=str(file.path), rule=LoadingFailureRule())]
+            return [MatchError(filename=file, rule=LoadingFailureRule())]
         if not yaml:
             return matches
 
         if isinstance(yaml, dict):
             yaml = [yaml]
 
-        yaml = ansiblelint.skip_utils.append_skipped_rules(yaml, file)
-
         for play in yaml:
-
             # Bug #849
             if play is None:
                 continue
 
-            if self.id in play.get("skipped_rules", ()):
+            if self.id in play.get(SKIPPED_RULES_KEY, ()):
+                continue
+
+            if "skip_ansible_lint" in play.get("tags", []):
                 continue
 
             matches.extend(self.matchplay(file, play))
@@ -226,7 +251,7 @@ class TransformMixin:
         self,
         match: MatchError,
         lintable: Lintable,
-        data: Union[CommentedMap, CommentedSeq, str],
+        data: CommentedMap | CommentedSeq | str,
     ) -> None:
         """Transform ``data`` to try to fix the MatchError identified by this rule.
 
@@ -262,8 +287,8 @@ class TransformMixin:
 
     @staticmethod
     def seek(
-        yaml_path: List[Union[int, str]],
-        data: Union[MutableMapping[str, Any], MutableSequence[Any], str],
+        yaml_path: list[int | str],
+        data: MutableMapping[str, Any] | MutableSequence[Any] | str,
     ) -> Any:
         """Get the element identified by ``yaml_path`` in ``data``.
 
@@ -291,43 +316,44 @@ class TransformMixin:
         return target
 
 
-def is_valid_rule(rule: Any) -> bool:
-    """Check if given rule is valid or not."""
-    return issubclass(rule, AnsibleLintRule) and bool(rule.id) and bool(rule.shortdesc)
-
-
 # pylint: disable=too-many-nested-blocks
-def load_plugins(directory: str) -> Iterator[AnsibleLintRule]:
+def load_plugins(  # noqa: max-complexity: 11
+    dirs: list[str],
+) -> Iterator[AnsibleLintRule]:
     """Yield a rule class."""
-    for pluginfile in glob.glob(os.path.join(directory, "[A-Za-z]*.py")):
 
-        pluginname = os.path.basename(pluginfile.replace(".py", ""))
-        spec = importlib.util.spec_from_file_location(pluginname, pluginfile)
+    def all_subclasses(cls: type) -> set[type]:
+        return set(cls.__subclasses__()).union(
+            [s for c in cls.__subclasses__() for s in all_subclasses(c)]
+        )
 
-        # https://github.com/python/typeshed/issues/2793
-        if spec and isinstance(spec.loader, Loader):
+    orig_sys_path = sys.path.copy()
 
-            # load rule markdown documentation if found
-            help_md = ""
-            if spec.origin:
-                md_filename = os.path.splitext(spec.origin)[0] + ".md"
-                if os.path.exists(md_filename):
-                    with open(md_filename, encoding="utf-8") as f:
-                        help_md = f.read()
+    for directory in dirs:
+        if directory not in sys.path:
+            sys.path.append(str(directory))
 
-            module = importlib.util.module_from_spec(spec)
-            spec.loader.exec_module(module)
-            try:
-                for _, obj in inspect.getmembers(module):
-                    if inspect.isclass(obj) and is_valid_rule(obj):
-                        if help_md:
-                            obj.help = help_md
-                        yield obj()
+        # load all modules in the directory
+        for f in Path(directory).glob("*.py"):
+            if "__" not in f.stem and f.stem not in "conftest":
+                import_module(f"{f.stem}")
+    # restore sys.path
+    sys.path = orig_sys_path
 
-            except (TypeError, ValueError, AttributeError) as exc:
-                _logger.warning(
-                    "Skipped invalid rule from %s due to %s", pluginname, exc
-                )
+    rules: dict[str, BaseRule] = {}
+    for rule in all_subclasses(BaseRule):
+        # we do not return the rules that are not loaded from passed 'directory'
+        # or rules that do not have a valid id. For example, during testing
+        # python may load other rule classes, some outside the tested rule
+        # directories.
+        if getattr(rule, "id") and Path(inspect.getfile(rule)).parent.absolute() in [
+            Path(x).absolute() for x in dirs
+        ]:
+            if issubclass(rule, BaseRule) and rule.id not in rules:
+                rules[rule.id] = rule()
+    for rule in rules.values():  # type: ignore
+        if isinstance(rule, AnsibleLintRule) and bool(rule.id):
+            yield rule
 
 
 class RulesCollection:
@@ -335,60 +361,80 @@ class RulesCollection:
 
     def __init__(
         self,
-        rulesdirs: Optional[List[str]] = None,
+        rulesdirs: list[str] | None = None,
         options: Namespace = default_options,
+        profile_name: str | None = None,
+        conditional: bool = True,
     ) -> None:
         """Initialize a RulesCollection instance."""
         self.options = options
+        self.profile = []
+        if profile_name:
+            self.profile = PROFILES[profile_name]
         if rulesdirs is None:
             rulesdirs = []
         self.rulesdirs = expand_paths_vars(rulesdirs)
-        self.rules: List[BaseRule] = []
+        self.rules: list[BaseRule] = []
         # internal rules included in order to expose them for docs as they are
         # not directly loaded by our rule loader.
         self.rules.extend(
-            [RuntimeErrorRule(), AnsibleParserErrorRule(), LoadingFailureRule()]
+            [
+                RuntimeErrorRule(),
+                AnsibleParserErrorRule(),
+                LoadingFailureRule(),
+                WarningRule(),
+            ]
         )
-        for rulesdir in self.rulesdirs:
-            _logger.debug("Loading rules from %s", rulesdir)
-            for rule in load_plugins(rulesdir):
-                self.register(rule)
+        for rule in load_plugins(rulesdirs):
+            self.register(rule, conditional=conditional)
         self.rules = sorted(self.rules)
 
-    def register(self, obj: AnsibleLintRule) -> None:
+        # When we have a profile we unload some of the rules
+        # But we do include all rules when listing all rules or tags
+        if profile_name and not (self.options.list_rules or self.options.list_tags):
+            filter_rules_with_profile(self.rules, profile_name)
+
+    def register(self, obj: AnsibleLintRule, conditional: bool = False) -> None:
         """Register a rule."""
         # We skip opt-in rules which were not manually enabled.
         # But we do include opt-in rules when listing all rules or tags
         if any(
             [
+                not conditional,
+                self.profile,  # when profile is used we load all rules and filter later
                 "opt-in" not in obj.tags,
                 obj.id in self.options.enable_list,
-                self.options.listrules,
-                self.options.listtags,
+                self.options.list_rules,
+                self.options.list_tags,
             ]
         ):
+            obj._collection = self  # pylint: disable=protected-access
             self.rules.append(obj)
 
     def __iter__(self) -> Iterator[BaseRule]:
         """Return the iterator over the rules in the RulesCollection."""
-        return iter(self.rules)
+        return iter(sorted(self.rules))
+
+    def alphabetical(self) -> Iterator[BaseRule]:
+        """Return an iterator over the rules in the RulesCollection in alphabetical order."""
+        return iter(sorted(self.rules, key=lambda x: x.id))
 
     def __len__(self) -> int:
         """Return the length of the RulesCollection data."""
         return len(self.rules)
 
-    def extend(self, more: List[AnsibleLintRule]) -> None:
+    def extend(self, more: list[AnsibleLintRule]) -> None:
         """Combine rules."""
         self.rules.extend(more)
 
-    def run(
+    def run(  # noqa: max-complexity: 12
         self,
         file: Lintable,
-        tags: Optional[Set[str]] = None,
-        skip_list: Optional[List[str]] = None,
-    ) -> List[MatchError]:
+        tags: set[str] | None = None,
+        skip_list: list[str] | None = None,
+    ) -> list[MatchError]:
         """Run all the rules against the given lintable."""
-        matches: List[MatchError] = []
+        matches: list[MatchError] = []
         if tags is None:
             tags = set()
         if skip_list is None:
@@ -398,7 +444,7 @@ class RulesCollection:
             try:
                 if file.content is not None:  # loads the file content
                     pass
-            except (IOError, UnicodeDecodeError) as exc:
+            except (OSError, UnicodeDecodeError) as exc:
                 return [
                     MatchError(
                         message=str(exc),
@@ -409,6 +455,8 @@ class RulesCollection:
                 ]
 
         for rule in self.rules:
+            if rule.id == "syntax-check":
+                continue
             if (
                 not tags
                 or rule.has_dynamic_tags
@@ -431,7 +479,7 @@ class RulesCollection:
             [rule.verbose() for rule in sorted(self.rules, key=lambda x: x.id)]
         )
 
-    def listtags(self) -> str:
+    def list_tags(self) -> str:
         """Return a string with all the tags in the RulesCollection."""
         tag_desc = {
             "command-shell": "Specific to use of command and shell modules",
@@ -466,19 +514,36 @@ class RulesCollection:
         return result
 
 
-def filter_rules_with_profile(rule_col: RulesCollection, profile: str) -> None:
+def filter_rules_with_profile(rule_col: list[BaseRule], profile: str) -> None:
     """Unload rules that are not part of the specified profile."""
     included = set()
     extends = profile
+    total_rules = len(rule_col)
     while extends:
         for rule in PROFILES[extends]["rules"]:
+            _logger.debug("Activating rule `%s` due to profile `%s`", rule, extends)
             included.add(rule)
         extends = PROFILES[extends].get("extends", None)
-    for rule in list(rule_col.rules):
+    for rule in rule_col.copy():
         if rule.id not in included:
             _logger.debug(
                 "Unloading %s rule due to not being part of %s profile.",
                 rule.id,
                 profile,
             )
-            rule_col.rules.remove(rule)
+            rule_col.remove(rule)
+        else:
+            for tag in ("opt-in", "experimental"):
+                if tag in rule.tags:
+                    _logger.debug(
+                        "Removing tag `%s` from `%s` rule because `%s` profile makes it mandatory.",
+                        tag,
+                        rule.id,
+                        profile,
+                    )
+                    rule.tags.remove(tag)
+                    # rule_col.rules.remove(rule)
+                    # break
+            if "opt-in" in rule.tags:
+                rule.tags.remove("opt-in")
+    _logger.debug("%s/%s rules included in the profile", len(rule_col), total_rules)
